# scraper.abstract_doc.core (auto generate by build_inits.py)
# flake8: noqa: F408

from scraper.abstract_doc.core.AbstractDocBase import AbstractDocBase
from scraper.abstract_doc.core.AbstractDocBasePathsMixin import \
    AbstractDocBasePathsMixin
from scraper.abstract_doc.core.AbstractDocGeneratorMixin import \
    AbstractDocGeneratorMixin
from scraper.abstract_doc.core.AbstractDocMetadataMixin import \
    AbstractDocMetadataMixin
from scraper.abstract_doc.core.AbstractDocRemotePathMixin import \
    AbstractDocRemotePathMixin
from scraper.abstract_doc.core.AbstractDocTextMixin import AbstractDocTextMixin



# scraper.abstract_doc (auto generate by build_inits.py)
# flake8: noqa: F408

from scraper.abstract_doc.AbstractDoc import AbstractDoc
from scraper.abstract_doc.core import (AbstractDocBase,
                                       AbstractDocBasePathsMixin,
                                       AbstractDocGeneratorMixin,
                                       AbstractDocMetadataMixin,
                                       AbstractDocRemotePathMixin,
                                       AbstractDocTextMixin)
from scraper.abstract_doc.data_mixins import AbstractWorksheetsMixin
from scraper.abstract_doc.hugging_face import AbstractDocHuggingFaceMixin
from scraper.abstract_doc.pipeline import (
    AbstractDocPipelineCleanupMixin, AbstractDocPipelineExtendedDataMixin,
    AbstractDocPipelineMetadataMixin, AbstractDocPipelineMixin)
from scraper.abstract_doc.readme import (
    AbstractDocChartDocsByTimeAndLangMixin, AbstractDocReadMeMixin,
    AbstractDocSummaryMixin)



import os
from functools import cached_property


class AbstractDocBasePathsMixin:
    @classmethod
    def get_main_branch_dir_root(cls) -> str:
        # e.g. lk_acts
        return "."

    @classmethod
    def get_data_branch_dir_root(cls) -> str:
        # e.g. ../lk_acts_data
        repo_name = cls.get_repo_name()
        return os.path.join(
            "..",
            f"{repo_name}_data",
        )

    @classmethod
    def get_data_branch_dir_root_data(cls) -> str:
        # e.g. ../lk_acts_data/data
        return os.path.join(
            cls.get_data_branch_dir_root(),
            "data",
        )

    @classmethod
    def get_dir_docs_for_cls_relative(cls) -> str:
        # e.g. data/lk_acts
        return os.path.join(
            "data",
            cls.get_doc_class_label(),
        )

    @classmethod
    def get_dir_docs_for_cls(cls) -> str:
        # e.g. ../lk_acts_data/data/lk_acts
        return os.path.join(
            cls.get_data_branch_dir_root_data(),
            cls.get_doc_class_label(),
        )

    @cached_property
    def dir_doc_relative_to_class(self) -> str:
        # e.g. 1990s/1995/ACT_1995_01
        return os.path.join(
            self.decade,
            self.year,
            self.doc_id,
        )

    @cached_property
    def dir_doc(self) -> str:
        # e.g. ../lk_acts_data/data/lk_acts/1990s/1995/ACT_1995_01
        return os.path.join(
            self.__class__.get_dir_docs_for_cls(),
            self.dir_doc_relative_to_class,
        )

    @cached_property
    def json_path(self) -> str:
        # e.g. ../lk_acts_data/data/lk_acts/1990s/1995/ACT_1995_01/doc.json
        return os.path.join(self.dir_doc, "doc.json")

    @property
    def has_metadata(self) -> bool:
        return os.path.exists(self.json_path)



import os
import re
from abc import ABC
from dataclasses import dataclass
from functools import cached_property

from utils import Hash, Log

log = Log("AbstractDocBase")


@dataclass
class AbstractDocBase(ABC):
    num: str
    date_str: str
    description: str
    url_metadata: str
    lang: str

    LANGS = ["si", "ta", "en", "si-ta-en"]

    @classmethod
    def get_repo_name(cls) -> str:
        return os.path.basename(os.getcwd())

    @classmethod
    def get_doc_class_label(cls) -> str:
        # If the repo has multiple document classes, then override this method
        return cls.get_repo_name()

    @classmethod
    def is_multi_doc(cls) -> bool:
        return cls.get_repo_name() != cls.get_doc_class_label()

    @classmethod
    def get_doc_class_description(cls) -> str:
        return f"A collection of {cls.get_doc_class_label()} documents."

    @classmethod
    def get_doc_class_emoji(cls) -> str:
        return "📄"

    @cached_property
    def num_short(self):
        if len(self.num) < 32:
            return self.num
        h = Hash.md5(self.num)
        return f"{self.num[:23]}-{h[:8]}"

    @cached_property
    def doc_id(self):
        doc_id = f"{self.date_str}-{self.num_short}"
        doc_id = re.sub(r"[^a-zA-Z0-9\-]", "-", doc_id)
        return doc_id

    @cached_property
    def decade(self) -> str:
        assert len(self.date_str) == 10
        return self.date_str[:3] + "0s"

    @cached_property
    def year(self) -> str:
        assert len(self.date_str) == 10
        return self.date_str[:4]



# scraper (auto generate by build_inits.py)
# flake8: noqa: F408

from scraper.abstract_doc import (AbstractDoc, AbstractDocBase,
                                  AbstractDocBasePathsMixin,
                                  AbstractDocChartDocsByTimeAndLangMixin,
                                  AbstractDocGeneratorMixin,
                                  AbstractDocHuggingFaceMixin,
                                  AbstractDocMetadataMixin,
                                  AbstractDocPipelineCleanupMixin,
                                  AbstractDocPipelineExtendedDataMixin,
                                  AbstractDocPipelineMetadataMixin,
                                  AbstractDocPipelineMixin,
                                  AbstractDocReadMeMixin,
                                  AbstractDocRemotePathMixin,
                                  AbstractDocSummaryMixin,
                                  AbstractDocTextMixin,
                                  AbstractWorksheetsMixin)
from scraper.abstract_excel_spreadsheet import AbstractExcelSpreadsheet
from scraper.abstract_pdf_doc import AbstractPDFDoc
from scraper.global_readme import GlobalReadMe, GlobalReadMeSummaryMixin



# scraper.abstract_doc.pipeline (auto generate by build_inits.py)
# flake8: noqa: F408

from scraper.abstract_doc.pipeline.AbstractDocPipelineCleanupMixin import \
    AbstractDocPipelineCleanupMixin
from scraper.abstract_doc.pipeline.AbstractDocPipelineExtendedDataMixin import \
    AbstractDocPipelineExtendedDataMixin
from scraper.abstract_doc.pipeline.AbstractDocPipelineMetadataMixin import \
    AbstractDocPipelineMetadataMixin
from scraper.abstract_doc.pipeline.AbstractDocPipelineMixin import \
    AbstractDocPipelineMixin



import requests
from utils import Log

log = Log("GlobalReadMeSummaryMixin")


class GlobalReadMeSummaryMixin:

    @classmethod
    def get_url_summary(cls, repo_name: str, doc_class_label: str) -> str:
        is_multi_doc = repo_name != doc_class_label
        branch_name = f"data_{doc_class_label}" if is_multi_doc else "data"
        return "/".join(
            [
                "https://raw.githubusercontent.com",
                "nuuuwan",
                repo_name,
                "refs",
                "heads",
                branch_name,
                "data",
                doc_class_label,
                "summary.json",
            ]
        )

    @classmethod
    def get_summary(cls, repo_name, doc_class_label):
        try:
            url = cls.get_url_summary(repo_name, doc_class_label)
            response = requests.get(url, timeout=120)
            response.raise_for_status()
            return response.json()
        except Exception as e:
            log.error(
                f"Failed to get summary for {repo_name}/{doc_class_label}: {e}"
            )
            return None

    def get_summary_list(self) -> list[dict]:
        summary_list = []
        for repo_name, doc_class_labels in self.repo_to_doc_classes.items():
            for doc_class_label in doc_class_labels:
                summary = self.get_summary(repo_name, doc_class_label)
                if summary:
                    summary_list.append(summary)
        return summary_list

    @classmethod
    def get_global_summary(cls, summary_list) -> dict:
        return dict(
            n_datasets=len(summary_list),
            n_docs=sum([s.get("n_docs", 0) for s in summary_list]),
            all_dataset_size=sum(
                [s.get("dataset_size", 0) for s in summary_list]
            ),
        )



import time
from multiprocessing import cpu_count

from utils import Log

log = Log("AbstractDocPipelineMetadataMixin")


class AbstractDocPipelineMetadataMixin:
    BATCH_SIZE = 8
    MAX_THREADS = min(4, cpu_count())

    @staticmethod
    def __log_processed_doc__(docs, dt):
        n_docs = len(docs)
        log.info(f"🛑 Processed {n_docs:,} docs in {dt:,.1f}s")

    @classmethod
    def gen_doc_batches_for_scrape_all_metadata(cls):
        current_batch = []
        for doc in cls.gen_docs():
            current_batch.append(doc)
            if len(current_batch) >= cls.BATCH_SIZE:
                yield current_batch
                current_batch = []

        if current_batch:
            yield current_batch

    @classmethod
    def process_batch_for_scrape_all_metadata(cls, doc_batch):
        for doc in doc_batch:
            doc.write()
        return doc_batch

    @classmethod
    def scrape_all_metadata(cls, max_dt):
        log.debug(f"BATCH_SIZE={cls.BATCH_SIZE}")
        log.debug(f"MAX_THREADS={cls.MAX_THREADS}")
        t_start = time.time()
        docs = []
        dt = 0
        for doc_batch in cls.gen_doc_batches_for_scrape_all_metadata():
            processed_doc_list = cls.process_batch_for_scrape_all_metadata(
                doc_batch
            )
            docs.extend(processed_doc_list)

            dt = time.time() - t_start
            if dt > max_dt:
                cls.__log_processed_doc__(docs, dt)
                log.info(f"🛑 Stopping. {dt:,.1f}s > {max_dt:,}s")
                return
        cls.__log_processed_doc__(docs, dt)
        log.info("🛑 All docs processed.")



import os
import tempfile
from abc import ABC
from dataclasses import dataclass
from functools import cached_property

import camelot
from utils import WWW, File, JSONFile, Log, PDFFile

from scraper.abstract_doc import AbstractDoc
from scraper.abstract_doc.data_mixins.AbstractWorksheetsMixin import (
    AbstractWorksheetsMixin,
)

log = Log("AbstractPDFDoc")


@dataclass
class AbstractPDFDoc(AbstractDoc, ABC, AbstractWorksheetsMixin):
    url_pdf: str

    # ----------------------------------------------------------------
    # PDF
    # ----------------------------------------------------------------
    @cached_property
    def pdf_path(self) -> str:
        return os.path.join(self.dir_doc, "doc.pdf")

    @property
    def has_pdf(self) -> bool:
        return os.path.exists(self.pdf_path)

    def download_pdf(self):
        with tempfile.NamedTemporaryFile(
            suffix=".pdf", delete=False
        ) as temp_pdf_f:
            temp_pdf_path = temp_pdf_f.name
            WWW(self.url_pdf).download_binary(temp_pdf_path)
            try:
                PDFFile(temp_pdf_path).compress(self.pdf_path)
            except Exception as e:
                log.warning(f"Failed to compress PDF from {self.url_pdf}: {e}")
                os.rename(temp_pdf_path, self.pdf_path)

    # ----------------------------------------------------------------
    # Blocks (Extracted from PDF)
    # ----------------------------------------------------------------
    @cached_property
    def blocks_path(self) -> str:
        return os.path.join(self.dir_doc, "blocks.json")

    @property
    def has_blocks(self) -> bool:
        return os.path.exists(self.blocks_path)

    def extract_blocks(self):
        if not self.has_pdf:
            return
        pdf_file = PDFFile(self.pdf_path)
        blocks = pdf_file.get_blocks()
        JSONFile(self.blocks_path).write(blocks)
        log.info(f"Wrote {self.blocks_path} ({len(blocks):,} blocks)")

    def get_blocks(self):
        return JSONFile(self.blocks_path).read()

    # ----------------------------------------------------------------
    # Worksheets (extracted from PDF)
    # ----------------------------------------------------------------

    def extract_worksheets(self):
        if not self.has_pdf:
            return
        tables = camelot.read_pdf(self.pdf_path, flavor="stream", pages="all")
        n_tables = len(tables)
        log.debug(f"Found {n_tables} tables in {self.pdf_path}")
        os.makedirs(self.dir_worksheets, exist_ok=True)
        tables.export(os.path.join(self.dir_worksheets, "table.csv"), f="csv")

    # ----------------------------------------------------------------
    # Text (From Blocks)
    # ----------------------------------------------------------------

    def get_text_from_block(self):
        if not self.has_pdf:
            return None
        blocks = self.get_blocks()
        text_list = [block["text"] for block in blocks if block["text"]]
        content = "\n\n".join(text_list)
        return content

    def get_text_all(self):
        return "\n".join(
            [
                "==== BLOCKS ==== ",
                "",
                self.get_text_from_block(),
                "",
                "==== WORKSHEETS ==== ",
                "",
                self.get_text_from_worksheets(),
                "",
            ]
        )

    def extract_text(self):
        if not self.has_pdf:
            return None
        File(self.text_path).write(self.get_text_all())
        log.info(f"Wrote {self.text_path}")

    # ----------------------------------------------------------------
    # All
    # ----------------------------------------------------------------
    def scrape_extended_data_for_doc(self):
        if not self.has_pdf:
            self.download_pdf()

        if self.has_pdf and not self.has_blocks:
            self.extract_blocks()

        if self.has_pdf and not self.has_worksheets:
            self.extract_worksheets()

        if self.has_blocks and not self.has_text:
            self.extract_text()



# scraper.global_readme (auto generate by build_inits.py)
# flake8: noqa: F408

from scraper.global_readme.GlobalReadMe import GlobalReadMe
from scraper.global_readme.GlobalReadMeSummaryMixin import \
    GlobalReadMeSummaryMixin



import time
from typing import Generator

from utils import Log, Parallel

log = Log("AbstractDocPipelineExtendedDataMixin")


class AbstractDocPipelineExtendedDataMixin:

    @classmethod
    def gen_doc_batches_for_scrape_extended_data(
        cls,
    ) -> Generator[list, None, None]:
        all_docs = cls.list_all()
        for i in range(0, len(all_docs), cls.BATCH_SIZE):
            i_start = i
            i_end = min(i + cls.BATCH_SIZE, len(all_docs))
            yield all_docs[i_start:i_end]

    @classmethod
    def process_doc_batch_for_scrape_extended_data(cls, doc_batch):
        def process(doc):
            try:
                doc.scrape_extended_data_for_doc()
                return doc
            except Exception as e:
                log.error(f"Error scraping extended data for {doc}: {e}")
                raise e

        processed_doc_list = Parallel.map(
            process,
            doc_batch,
            max_threads=cls.MAX_THREADS,
        )
        processed_doc_list = [doc for doc in processed_doc_list if doc]
        return processed_doc_list

    @classmethod
    def scrape_all_extended_data(cls, max_dt):
        log.debug(f"BATCH_SIZE={cls.BATCH_SIZE}")
        log.debug(f"MAX_THREADS={cls.MAX_THREADS}")

        t_start = time.time()
        for doc_batch in cls.gen_doc_batches_for_scrape_extended_data():
            cls.process_doc_batch_for_scrape_extended_data(doc_batch)
            dt = time.time() - t_start
            if dt > max_dt:
                log.info(f"🛑 Stopping. {dt:,.1f}s > {max_dt:,}s")
                return
        log.info("🛑 All extended data scraped.")

    def scrape_extended_data_for_doc(self):
        pass



import json
import os

from utils import File, FileOrDirectory, Format, Log

log = Log("AbstractDocReadMeMixin")


class AbstractDocReadMeMixin:
    N_LATEST = 20
    URL_ARXIV = "https://arxiv.org/abs/2510.04124"

    @classmethod
    def get_lines_for_header(cls, summary) -> list[str]:
        title = Format.title(
            summary["doc_class_emoji"] + summary["doc_class_label"]
        )
        return [
            f"# {title} `Dataset`",
            "",
        ]

    @classmethod
    def get_line_for_blurb_item_files(cls, summary) -> list[str]:
        n_docs = summary["n_docs"]
        n_docs_with_pdfs = summary["n_docs_with_pdfs"]
        n_docs_with_text = summary["n_docs_with_text"]
        n_docs_with_excel = summary.get("n_docs_with_excel", 0)

        blob_list = []
        for doc_type, n in [
            ("JSON", n_docs),
            ("PDF", n_docs_with_pdfs),
            ("TXT", n_docs_with_text),
            ("Excel", n_docs_with_excel),
            ("🤗 Hugging Face", n_docs_with_text),
            ("Something New", 0),
        ]:
            if n == 0:
                continue
            p = n / n_docs

            label = "" if (n == n_docs) else f"({p:.0%})"
            blob_list.append(f"**{doc_type}** {label}".strip())
        return "- In " + Format.and_list(blob_list)

    @classmethod
    def get_line_for_blurb_item_lang(cls, summary) -> list[str]:
        langs = summary["langs"]
        blurb_langs = []
        for lang_i, lang_label in [
            ["si", "සිංහල"],
            ["ta", "தமிழ்"],
            ["en", "English"],
        ]:
            for lang in langs:
                if lang_i in lang:
                    blurb_langs.append(f"**{lang_label}**")
                    break

        return "- In " + Format.and_list(blurb_langs)

    @classmethod
    def render_url_source_list(cls, url_source_list) -> str:
        return Format.and_list([f"[{url}]({url})" for url in url_source_list])

    @classmethod
    def get_lines_for_blurb(cls, summary) -> list[str]:
        doc_class_description = summary["doc_class_description"]
        time_updated = summary["time_updated"]
        n_docs = summary["n_docs"]
        date_str_min = summary["date_str_min"]
        date_str_max = summary["date_str_max"]
        dataset_size = summary["dataset_size"]
        url_data = summary["url_data"]

        dataset_size_humanized = FileOrDirectory.humanize_size(dataset_size)
        time_updated_for_badge = Format.badge(time_updated)

        url_source_list = summary.get("url_source_list") or [
            summary["url_source"]
        ]  # HACK! legacy url_source should be removed

        return [
            "![LastUpdated](https://img.shields.io/badge"
            + f"/last_updated-{time_updated_for_badge}-green)",
            "",
            f"[{url_data}]({url_data})",
            "",
            doc_class_description,
            "",
            f"- [**{n_docs:,}** documents]({url_data})"
            + f" (**{dataset_size_humanized}**),"
            + f" from **{date_str_min}** to **{date_str_max}**,"
            + " scraped from "
            + cls.render_url_source_list(url_source_list),
            "",
            cls.get_line_for_blurb_item_files(summary),
            "",
            cls.get_line_for_blurb_item_lang(summary),
            "",
            f"- 🎓 Cite as **[arXiv:2510.04124]({cls.URL_ARXIV}) [cs.CL]**",
            "",
        ]

    @classmethod
    def get_lines_for_metadata_example(cls, summary) -> list[str]:
        latest_doc_d = summary["latest_doc_d"]
        return [
            "## 📝 Example Metadata",
            "",
            "```json",
            json.dumps(latest_doc_d, indent=4),
            "```",
            "",
        ]

    @classmethod
    def get_lines_chart_docs_by_year_and_lang(cls, summary) -> list[str]:
        url_chart = summary["url_chart"]
        return [
            f"![Chart]({url_chart})",
            "",
        ]

    @classmethod
    def get_lines_for_hugging_face(cls):
        url_hf = (
            "https://img.shields.io/badge"
            + "/-HuggingFace-FDEE21"
            + "?style=for-the-badge&logo=HuggingFace"
        )
        lines = [
            "## 🤗 Hugging Face Datasets",
            "",
            f"![HuggingFace]({url_hf})",
            "",
        ]

        for label_suffix in ["docs", "chunks"]:
            dataset_id = cls.get_dataset_id(label_suffix)
            url = cls.get_dataset_url(label_suffix)
            lines.append(f"- [{dataset_id}]({url})")
        lines.append("")
        return lines

    @classmethod
    def get_lines_for_latest_docs(cls):
        lines = [f"## 🆕 {cls.N_LATEST} Latest documents", ""]
        for doc in cls.list_all()[: cls.N_LATEST]:
            line = "- " + " | ".join(
                [
                    doc.date_str,
                    f"`{doc.num}`",
                    doc.description,
                    f"[data]({doc.remote_data_url})",
                ]
            )
            lines.append(line)
        lines.append("")
        return lines

    @classmethod
    def get_lines_for_more_datasets(cls) -> list[str]:
        return [
            "---",
            "",
            "### [More Datasets about 🇱🇰 #SriLanka]"
            + "(https://github.com/nuuuwan/lk_datasets)",
            "",
        ]

    @classmethod
    def get_lines_for_footer(cls) -> list[str]:
        return [
            "![Maintainer]"
            + "(https://img.shields.io/badge/maintainer-nuuuwan-red)",
            "![MadeWith](https://img.shields.io/badge/made_with-python-blue)",
            "[![License: MIT]"
            + "(https://img.shields.io/badge/License-MIT-yellow.svg)]"
            + "(https://opensource.org/licenses/MIT)",
            "",
        ]

    @classmethod
    def lines(cls) -> list[str]:
        summary = cls.get_summary()
        return (
            cls.get_lines_for_header(summary)
            + cls.get_lines_for_blurb(summary)
            + cls.get_lines_for_metadata_example(summary)
            + cls.get_lines_chart_docs_by_year_and_lang(summary)
            + cls.get_lines_for_hugging_face()
            + cls.get_lines_for_latest_docs()
            + cls.get_lines_for_more_datasets()
            + cls.get_lines_for_footer()
        )

    @classmethod
    def get_doc_class_readme_path(cls) -> str:
        # E.g. ../lk_acts_data/data/lk_acts/README.md
        return os.path.join(cls.get_dir_docs_for_cls(), "README.md")

    @classmethod
    def build_doc_class_readme(cls):
        assert cls.list_all()
        os.makedirs(cls.get_main_branch_dir_root(), exist_ok=True)
        readme_path = cls.get_doc_class_readme_path()
        File(readme_path).write("\n".join(cls.lines()))
        log.info(f"Wrote {readme_path}")



"""Setup."""

import setuptools

DIST_NAME = "scraper"
VERSION = "1.1.1"
DESCRIPTION = "Utilities for Scraping Documents from Websites."
INSTALL_REQUIRES = [
    "utils-nuuuwan",
    "bs4",
    "pymupdf",
    "matplotlib",
    "datasets",
    "huggingface_hub",
    "pandas",
    "numpy",
    "selenium",
]

setuptools.setup(
    name="%s-nuuuwan" % DIST_NAME,
    version=VERSION,
    author="Nuwan I. Senaratna",
    author_email="nuuuwan@gmail.com",
    description=DESCRIPTION,
    long_description="",
    long_description_content_type="text/markdown",
    url="https://github.com/nuuuwan/%s" % DIST_NAME,
    project_urls={
        "Bug Tracker": "https://github.com/nuuuwan/%s/issues" % DIST_NAME,
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.10",
    install_requires=INSTALL_REQUIRES,
    test_suite="nose.collector",
    tests_require=["nose"],
)
